@@ -33,6 +33,7 @@ import argparse
 from configparser import ConfigParser
 import logging
 from gettext import ngettext
+from pathlib import Path
 
 from . import _
 from . import common
@@ -48,6 +49,9 @@ try:
 except ImportError:
     pass
 
+buildserverid = None
+ssh_channel = None
+
 
 # Note that 'force' here also implies test mode.
 def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
@@ -66,7 +70,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         target folder for the build result
     force
     """
-    global buildserverid
+    global buildserverid, ssh_channel
 
     try:
         paramiko
@@ -114,7 +118,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
         # Helper to copy the contents of a directory to the server...
         def send_dir(path):
-            logging.debug("rsyncing " + path + " to " + ftp.getcwd())
+            logging.debug("rsyncing %s to %s" % (path, ftp.getcwd()))
             # TODO this should move to `vagrant rsync` from >= v1.5
             try:
                 subprocess.check_output(['rsync', '--recursive', '--perms', '--links', '--quiet', '--rsh='
@@ -129,7 +133,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
                                          sshinfo['user'] + "@" + sshinfo['hostname'] + ":" + ftp.getcwd()],
                                         stderr=subprocess.STDOUT)
             except subprocess.CalledProcessError as e:
-                raise FDroidException(str(e), e.output.decode())
+                raise FDroidException(str(e), e.output.decode()) from e
 
         logging.info("Preparing server for build...")
         serverpath = os.path.abspath(os.path.dirname(__file__))
@@ -221,8 +225,8 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
         # Execute the build script...
         logging.info("Starting build...")
-        chan = sshs.get_transport().open_session()
-        chan.get_pty()
+        ssh_channel = sshs.get_transport().open_session()
+        ssh_channel.get_pty()
         cmdline = posixpath.join(homedir, 'fdroidserver', 'fdroid')
         cmdline += ' build --on-server'
         if force:
@@ -233,15 +237,17 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             cmdline += ' --skip-scan'
         if options.notarball:
             cmdline += ' --no-tarball'
+        if (options.scan_binary or config.get('scan_binary')) and not options.skipscan:
+            cmdline += ' --scan-binary'
         cmdline += " %s:%s" % (app.id, build.versionCode)
-        chan.exec_command('bash --login -c "' + cmdline + '"')  # nosec B601 inputs are sanitized
+        ssh_channel.exec_command('bash --login -c "' + cmdline + '"')  # nosec B601 inputs are sanitized
 
         # Fetch build process output ...
         try:
-            cmd_stdout = chan.makefile('rb', 1024)
+            cmd_stdout = ssh_channel.makefile('rb', 1024)
             output = bytes()
             output += common.get_android_tools_version_log().encode()
-            while not chan.exit_status_ready():
+            while not ssh_channel.exit_status_ready():
                 line = cmd_stdout.readline()
                 if line:
                     if options.verbose:
@@ -258,7 +264,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
         # Check build process exit status ...
         logging.info("...getting exit status")
-        returncode = chan.recv_exit_status()
+        returncode = ssh_channel.recv_exit_status()
         if returncode != 0:
             if timeout_event.is_set():
                 message = "Timeout exceeded! Build VM force-stopped for {0}:{1}"
@@ -288,10 +294,10 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             ftp.get(apkfile, os.path.join(output_dir, apkfile))
             if not options.notarball:
                 ftp.get(tarball, os.path.join(output_dir, tarball))
-        except Exception:
+        except Exception as exc:
             raise BuildException(
                 "Build failed for {0}:{1} - missing output files".format(
-                    app.id, build.versionName), str(output, 'utf-8'))
+                    app.id, build.versionName), str(output, 'utf-8')) from exc
         ftp.close()
 
     finally:
@@ -331,10 +337,6 @@ def transform_first_char(string, method):
     return method(string[0]) + string[1:]
 
 
-def add_failed_builds_entry(failed_builds, appid, build, entry):
-    failed_builds.append([appid, int(build.versionCode), str(entry)])
-
-
 def get_metadata_from_apk(app, build, apkfile):
     """Get the required metadata from the built APK.
 
@@ -386,7 +388,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
             logging.info("Running 'sudo' commands in %s" % os.getcwd())
 
             p = FDroidPopen(['sudo', 'DEBIAN_FRONTEND=noninteractive',
-                             'bash', '-x', '-c', build.sudo])
+                             'bash', '-e', '-u', '-o', 'pipefail', '-x', '-c', '; '.join(build.sudo)])
             if p.returncode != 0:
                 raise BuildException("Error running sudo command for %s:%s" %
                                      (app.id, build.versionName), p.output)
@@ -531,13 +533,13 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
     # Run a build command if one is required...
     if build.build:
         logging.info("Running 'build' commands in %s" % root_dir)
-        cmd = common.replace_config_vars(build.build, build)
+        cmd = common.replace_config_vars("; ".join(build.build), build)
 
         # Substitute source library paths into commands...
         for name, number, libpath in srclibpaths:
             cmd = cmd.replace('$$' + name + '$$', os.path.join(os.getcwd(), libpath))
 
-        p = FDroidPopen(['bash', '-x', '-c', cmd], cwd=root_dir)
+        p = FDroidPopen(['bash', '-e', '-u', '-o', 'pipefail', '-x', '-c', cmd], cwd=root_dir)
 
         if p.returncode != 0:
             raise BuildException("Error running build command for %s:%s" %
@@ -789,6 +791,23 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
             raise BuildException('No apks match %s' % globpath)
         src = os.path.normpath(apks[0])
 
+    # Run a postbuild command if one is required...
+    if build.postbuild:
+        logging.info(f"Running 'postbuild' commands in {root_dir}")
+        cmd = common.replace_config_vars("; ".join(build.postbuild), build)
+
+        # Substitute source library paths into commands...
+        for name, number, libpath in srclibpaths:
+            cmd = cmd.replace(f"$${name}$$", str(Path.cwd() / libpath))
+
+        cmd = cmd.replace('$$OUT$$', str(Path(src).resolve()))
+
+        p = FDroidPopen(['bash', '-e', '-u', '-o', 'pipefail', '-x', '-c', cmd], cwd=root_dir)
+
+        if p.returncode != 0:
+            raise BuildException("Error running postbuild command for "
+                                 f"{app.id}:{build.versionName}", p.output)
+
     # Make sure it's not debuggable...
     if common.is_apk_and_debuggable(src):
         raise BuildException("APK is debuggable")
@@ -803,10 +822,10 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
         vercode, version = get_metadata_from_apk(app, build, src)
         if version != build.versionName or vercode != build.versionCode:
             raise BuildException(("Unexpected version/version code in output;"
-                                  " APK: '%s' / '%s', "
-                                  " Expected: '%s' / '%s'")
-                                 % (version, str(vercode), build.versionName,
-                                    str(build.versionCode)))
+                                  " APK: '%s' / '%d', "
+                                  " Expected: '%s' / '%d'")
+                                 % (version, vercode, build.versionName,
+                                    build.versionCode))
         if (options.scan_binary or config.get('scan_binary')) and not options.skipscan:
             if scanner.scan_binary(src):
                 raise BuildException("Found blocklisted packages in final apk!")
@@ -892,8 +911,10 @@ def force_halt_build(timeout):
     """Halt the currently running Vagrant VM, to be called from a Timer."""
     logging.error(_('Force halting build after {0} sec timeout!').format(timeout))
     timeout_event.set()
+    if ssh_channel:
+        ssh_channel.close()
     vm = vmtools.get_build_vm('builder')
-    vm.halt()
+    vm.destroy()
 
 
 def parse_commandline():
@@ -951,7 +972,6 @@ def parse_commandline():
 
 options = None
 config = None
-buildserverid = None
 fdroidserverid = None
 start_timestamp = time.gmtime()
 status_output = None
@@ -984,19 +1004,13 @@ def main():
         if not options.appid and not options.all:
             parser.error("option %s: If you really want to build all the apps, use --all" % "all")
 
-    config = common.read_config(options)
+    config = common.read_config(opts=options)
 
     if config['build_server_always']:
         options.server = True
     if options.reset_server and not options.server:
         parser.error("option %s: Using --reset-server without --server makes no sense" % "reset-server")
 
-    if options.onserver or not options.server:
-        for d in ['build-tools', 'platform-tools', 'tools']:
-            if not os.path.isdir(os.path.join(config['sdk_path'], d)):
-                raise FDroidException(_("Android SDK '{path}' does not have '{dirname}' installed!")
-                                      .format(path=config['sdk_path'], dirname=d))
-
     log_dir = 'logs'
     if not os.path.isdir(log_dir):
         logging.info("Creating log directory")
@@ -1073,12 +1087,13 @@ def main():
                 app['Builds'] = [build]
                 break
 
+    if not options.onserver:
+        common.write_running_status_json(status_output)
+
     # Build applications...
     failed_builds = []
-    build_succeeded = []
     build_succeeded_ids = []
     status_output['failedBuilds'] = failed_builds
-    status_output['successfulBuilds'] = build_succeeded
     status_output['successfulBuildIds'] = build_succeeded_ids
     # Only build for 72 hours, then stop gracefully.
     endtime = time.time() + 72 * 60 * 60
@@ -1096,7 +1111,7 @@ def main():
             if build.timeout is None:
                 timeout = 7200
             else:
-                timeout = int(build.timeout)
+                timeout = build.timeout
             if options.server and timeout > 0:
                 logging.debug(_('Setting {0} sec timeout for this build').format(timeout))
                 timer = threading.Timer(timeout, force_halt_build, [timeout])
@@ -1106,9 +1121,6 @@ def main():
                 timer = None
 
             tools_version_log = ''
-            if not options.onserver:
-                tools_version_log = common.get_android_tools_version_log()
-                common.write_running_status_json(status_output)
             try:
 
                 # For the first build of a particular app, we need to set up
@@ -1118,8 +1130,7 @@ def main():
                     vcs, build_dir = common.setup_vcs(app)
                     first = False
 
-                logging.info("Using %s" % vcs.clientversion())
-                logging.debug("Checking " + build.versionName)
+                logging.debug("Checking %s:%s" % (appid, build.versionCode))
                 if trybuild(app, build, build_dir, output_dir, log_dir,
                             also_check_dir, srclib_dir, extlib_dir,
                             tmp_dir, repo_dir, vcs, options.test,
@@ -1132,7 +1143,7 @@ def main():
                             tools_version_log = ''.join(f.readlines())
                         os.remove(toolslog)
 
-                    if app.Binaries is not None:
+                    if url := build.binary or app.Binaries:
                         # This is an app where we build from source, and
                         # verify the APK contents against a developer's
                         # binary. We get that binary now, and save it
@@ -1144,7 +1155,6 @@ def main():
                                          "developer supplied reference "
                                          "binaries: '{path}'"
                                          .format(path=binaries_dir))
-                        url = app.Binaries
                         url = url.replace('%v', build.versionName)
                         url = url.replace('%c', str(build.versionCode))
                         logging.info("...retrieving " + url)
@@ -1194,9 +1204,11 @@ def main():
                                              'supplied reference binary '
                                              'successfully')
 
-                    build_succeeded.append(app)
                     build_succeeded_ids.append([app['id'], build.versionCode])
 
+                    if not options.onserver:
+                        common.write_running_status_json(status_output)
+
             except VCSException as vcse:
                 reason = str(vcse).split('\n', 1)[0] if options.verbose else str(vcse)
                 logging.error("VCS error while building app %s: %s" % (
@@ -1204,10 +1216,13 @@ def main():
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
-                add_failed_builds_entry(failed_builds, appid, build, vcse)
+                failed_builds.append((appid, build.versionCode))
                 common.deploy_build_log_with_rsync(
                     appid, build.versionCode, "".join(traceback.format_exc())
                 )
+                if not options.onserver:
+                    common.write_running_status_json(status_output)
+
             except FDroidException as e:
                 tstamp = time.strftime("%Y-%m-%d %H:%M:%SZ", time.gmtime())
                 with open(os.path.join(log_dir, appid + '.log'), 'a+') as f:
@@ -1222,20 +1237,25 @@ def main():
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
-                add_failed_builds_entry(failed_builds, appid, build, e)
+                failed_builds.append((appid, build.versionCode))
                 common.deploy_build_log_with_rsync(
                     appid, build.versionCode, "".join(traceback.format_exc())
                 )
-            except Exception as e:
+                if not options.onserver:
+                    common.write_running_status_json(status_output)
+
+            except Exception:
                 logging.error("Could not build app %s due to unknown error: %s" % (
                     appid, traceback.format_exc()))
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
-                add_failed_builds_entry(failed_builds, appid, build, e)
+                failed_builds.append((appid, build.versionCode))
                 common.deploy_build_log_with_rsync(
                     appid, build.versionCode, "".join(traceback.format_exc())
                 )
+                if not options.onserver:
+                    common.write_running_status_json(status_output)
 
             if timer:
                 timer.cancel()  # kill the watchdog timer
@@ -1245,17 +1265,17 @@ def main():
             logging.info("Stopping after global build timeout...")
             break
 
-    for app in build_succeeded:
-        logging.info("success: %s" % (app.id))
+    for app in build_succeeded_ids:
+        logging.info("success: %s" % app[0])
 
     if not options.verbose:
         for fb in failed_builds:
-            logging.info('Build for app {}:{} failed:\n{}'.format(*fb))
+            logging.info('Build for app {}:{} failed'.format(*fb))
 
     logging.info(_("Finished"))
-    if len(build_succeeded) > 0:
+    if len(build_succeeded_ids) > 0:
         logging.info(ngettext("{} build succeeded",
-                              "{} builds succeeded", len(build_succeeded)).format(len(build_succeeded)))
+                              "{} builds succeeded", len(build_succeeded_ids)).format(len(build_succeeded_ids)))
     if len(failed_builds) > 0:
         logging.info(ngettext("{} build failed",
                               "{} builds failed", len(failed_builds)).format(len(failed_builds)))
@@ -1270,8 +1290,7 @@ def main():
                     if m:
                         status_output['hostProcMeminfoMemTotal'] = m.group(1)
                         break
-        fdroid_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))
-        buildserver_config = os.path.join(fdroid_path, 'makebuildserver.config.py')
+        buildserver_config = 'builder/Vagrantfile'
         if os.path.isfile(buildserver_config) and os.access(buildserver_config, os.R_OK):
             with open(buildserver_config) as configfile:
                 for line in configfile:
