@@ -23,14 +23,11 @@ from pathlib import Path
 import platform
 import re
 import logging
-import yaml
-try:
-    from yaml import CSafeLoader as SafeLoader
-except ImportError:
-    from yaml import SafeLoader
 import importlib
 from collections import OrderedDict
 
+from ruamel.yaml import YAML, YAMLError
+
 from . import common
 from . import _
 from .exception import MetaDataException, FDroidException
@@ -72,7 +69,6 @@ yaml_app_field_order = [
     'Donate',
     'FlattrID',
     'Liberapay',
-    'LiberapayID',
     'OpenCollective',
     'Bitcoin',
     'Litecoin',
@@ -135,7 +131,6 @@ class App(dict):
         self.Donate = None
         self.FlattrID = None
         self.Liberapay = None
-        self.LiberapayID = None
         self.OpenCollective = None
         self.Bitcoin = None
         self.Litecoin = None
@@ -153,7 +148,7 @@ class App(dict):
         self.AutoUpdateMode = 'None'
         self.UpdateCheckMode = 'None'
         self.UpdateCheckIgnore = None
-        self.VercodeOperation = None
+        self.VercodeOperation = []
         self.UpdateCheckName = None
         self.UpdateCheckData = None
         self.CurrentVersion = ''
@@ -203,7 +198,9 @@ fieldtypes = {
     'Categories': TYPE_LIST,
     'AntiFeatures': TYPE_LIST,
     'AllowedAPKSigningKeys': TYPE_LIST,
-    'Build': TYPE_BUILD,
+    'Builds': TYPE_BUILD,
+    'VercodeOperation': TYPE_LIST,
+    'CurrentVersionCode': TYPE_INT,
 }
 
 
@@ -230,6 +227,7 @@ build_flags = [
     'maven',
     'buildozer',
     'output',
+    'binary',
     'srclibs',
     'oldsdkloc',
     'encoding',
@@ -248,6 +246,7 @@ build_flags = [
     'preassemble',
     'gradleprops',
     'antcommands',
+    'postbuild',
     'novcheck',
     'antifeatures',
 ]
@@ -269,6 +268,7 @@ class Build(dict):
         self.maven = False
         self.buildozer = False
         self.output = None
+        self.binary = None
         self.srclibs = []
         self.oldsdkloc = False
         self.encoding = None
@@ -287,6 +287,7 @@ class Build(dict):
         self.preassemble = []
         self.gradleprops = []
         self.antcommands = []
+        self.postbuild = ''
         self.novcheck = False
         self.antifeatures = []
         if copydict:
@@ -351,6 +352,7 @@ flagtypes = {
     'init': TYPE_SCRIPT,
     'prebuild': TYPE_SCRIPT,
     'build': TYPE_SCRIPT,
+    'postbuild': TYPE_SCRIPT,
     'submodules': TYPE_BOOL,
     'oldsdkloc': TYPE_BOOL,
     'forceversion': TYPE_BOOL,
@@ -405,10 +407,6 @@ valuetypes = {
                    VALID_USERNAME_REGEX,
                    ['Liberapay']),
 
-    FieldValidator("Liberapay ID",
-                   r'^[0-9]+$',
-                   ['LiberapayID']),
-
     FieldValidator("Open Collective",
                    VALID_USERNAME_REGEX,
                    ['OpenCollective']),
@@ -481,18 +479,19 @@ def parse_yaml_srclib(metadatapath):
 
     with metadatapath.open("r", encoding="utf-8") as f:
         try:
-            data = yaml.load(f, Loader=SafeLoader)
+            yaml = YAML(typ='safe')
+            data = yaml.load(f)
             if type(data) is not dict:
                 if platform.system() == 'Windows':
                     # Handle symlink on Windows
                     symlink = metadatapath.parent / metadatapath.read_text(encoding='utf-8')
                     if symlink.is_file():
                         with symlink.open("r", encoding="utf-8") as s:
-                            data = yaml.load(s, Loader=SafeLoader)
+                            data = yaml.load(s)
             if type(data) is not dict:
-                raise yaml.error.YAMLError(_('{file} is blank or corrupt!')
-                                           .format(file=metadatapath))
-        except yaml.error.YAMLError as e:
+                raise YAMLError(_('{file} is blank or corrupt!')
+                                .format(file=metadatapath))
+        except YAMLError as e:
             _warn_or_exception(_("Invalid srclib metadata: could not "
                                  "parse '{file}'")
                                .format(file=metadatapath) + '\n'
@@ -514,8 +513,11 @@ def parse_yaml_srclib(metadatapath):
                     thisinfo[key] = data[key]
                 elif data[key] is None:
                     thisinfo[key] = ['']
-            elif key == 'Prepare' and isinstance(data[key], list):
-                thisinfo[key] = ' && '.join(data[key])
+            elif key == 'Prepare' or flagtype(key) == TYPE_SCRIPT:
+                if isinstance(data[key], list):
+                    thisinfo[key] = data[key]
+                else:
+                    thisinfo[key] = [data[key]] if data[key] else []
             else:
                 thisinfo[key] = str(data[key] or '')
 
@@ -615,29 +617,31 @@ def split_list_values(s):
 
 
 def sorted_builds(builds):
-    return sorted(builds, key=lambda build: int(build.versionCode))
+    return sorted(builds, key=lambda build: build.versionCode)
 
 
 esc_newlines = re.compile(r'\\( |\n)')
 
 
 def post_metadata_parse(app):
-    # TODO keep native types, convert only for .txt metadata
-    for k, v in app.items():
-        if type(v) in (float, int):
-            app[k] = str(v)
-
     if 'flavours' in app and app['flavours'] == [True]:
         app['flavours'] = 'yes'
 
-    for field, fieldtype in fieldtypes.items():
-        if fieldtype != TYPE_LIST:
-            continue
-        value = app.get(field)
-        if isinstance(value, str):
-            app[field] = [value, ]
-        elif value is not None:
-            app[field] = [str(i) for i in value]
+    for k, v in app.items():
+        if fieldtype(k) == TYPE_LIST:
+            if isinstance(v, str):
+                app[k] = [v, ]
+            elif v:
+                app[k] = [str(i) for i in v]
+        elif fieldtype(k) == TYPE_INT:
+            if v:
+                app[k] = int(v)
+        elif fieldtype(k) == TYPE_STRING:
+            if v:
+                app[k] = str(v)
+        else:
+            if type(v) in (float, int):
+                app[k] = str(v)
 
     def _yaml_bool_unmapable(v):
         return v in (True, False, [True], [False])
@@ -673,7 +677,7 @@ def post_metadata_parse(app):
                             else:
                                 build[k] = []
                     elif flagtype(k) is TYPE_INT:
-                        build[k] = str(v)
+                        build[k] = v
                     elif flagtype(k) is TYPE_STRING:
                         if isinstance(v, bool) and k in _bool_allowed:
                             build[k] = v
@@ -753,8 +757,7 @@ def parse_metadata(metadatapath):
         metadata_in_repo = build_dir / '.fdroid.yml'
         if metadata_in_repo.is_file():
             try:
-                # TODO: Python3.6: Should accept path-like
-                commit_id = common.get_head_commit_id(git.Repo(str(build_dir)))
+                commit_id = common.get_head_commit_id(git.Repo(build_dir))
                 logging.debug(_('Including metadata from %s@%s') % (metadata_in_repo, commit_id))
             except git.exc.InvalidGitRepositoryError:
                 logging.debug(_('Including metadata from {path}').format(metadata_in_repo))
@@ -790,8 +793,9 @@ def parse_yaml_metadata(mf, app):
 
     """
     try:
-        yamldata = yaml.load(mf, Loader=SafeLoader)
-    except yaml.YAMLError as e:
+        yaml = YAML(typ='safe')
+        yamldata = yaml.load(mf)
+    except YAMLError as e:
         _warn_or_exception(_("could not parse '{path}'")
                            .format(path=mf.name) + '\n'
                            + common.run_yamllint(mf.name, indent=4),
@@ -816,7 +820,7 @@ def parse_yaml_metadata(mf, app):
                                   "metadata because it is deprecated.")
                                 .format(field=deprecated_field,
                                         metapath=mf.name))
-                del(yamldata[deprecated_field])
+                del yamldata[deprecated_field]
 
         if yamldata.get('Builds', None):
             for build in yamldata.get('Builds', []):
@@ -843,9 +847,8 @@ def post_parse_yaml_metadata(yamldata):
             _flagtype = flagtype(flag)
 
             if _flagtype is TYPE_SCRIPT:
-                # concatenate script flags into a single string if they are stored as list
-                if isinstance(build[flag], list):
-                    build[flag] = ' && '.join(build[flag])
+                if isinstance(build[flag], str):
+                    build[flag] = [build[flag]]
             elif _flagtype is TYPE_STRING:
                 # things like versionNames are strings, but without quotes can be numbers
                 if isinstance(build[flag], float) or isinstance(build[flag], int):
@@ -912,12 +915,6 @@ def write_yaml(mf, app):
                     return value[0]
                 else:
                     return value
-            else:
-                script_lines = value.split(' && ')
-                if len(script_lines) > 1:
-                    return script_lines
-                else:
-                    return value
         else:
             return value
 
@@ -962,16 +959,17 @@ def write_yaml(mf, app):
                 build = Build(build)
             b = ruamel.yaml.comments.CommentedMap()
             for field in build_flags:
-                value = getattr(build, field)
-                if hasattr(build, field) and value:
+                if hasattr(build, field):
+                    value = getattr(build, field)
                     if field == 'gradle' and value == ['off']:
                         value = [ruamel.yaml.scalarstring.SingleQuotedScalarString('off')]
                     if field in ('maven', 'buildozer'):
                         if value == 'no':
                             continue
-                        elif value == 'yes':
-                            value = 'yes'
-                    b.update({field: _field_to_yaml(flagtype(field), value)})
+                    typ = flagtype(field)
+                    # don't check value == True for TYPE_INT as it could be 0
+                    if value is not None and (typ == TYPE_INT or value):
+                        b.update({field: _field_to_yaml(typ, value)})
             builds.append(b)
 
         # insert extra empty lines between build entries
