@@ -1,12 +1,9 @@
 
+# TODO convert to config.yml!
+
 repo_url = "https://MyFirstFDroidRepo.org/fdroid/repo"
 repo_name = "My First F-Droid Repo Demo"
-repo_description = """
-This is a repository of apps to be used with F-Droid. Applications in this
-repository are either official binaries built by the original application
-developers, or are binaries built from source by the admin of f-droid.org
-using the tools on https://gitlab.com/u/fdroid.
-"""
+repo_description = """This is a repository of apps to be used with F-Droid. Applications in this repository are either official binaries built by the original application developers, or are binaries built from source by the admin of f-droid.org using the tools on https://gitlab.com/fdroid."""
 
 archive_older = 3
 archive_url = "https://f-droid.org/archive"
@@ -32,3 +29,5 @@ update_stats = True
 
 install_list = 'org.adaway'
 uninstall_list = ('com.android.vending', 'com.facebook.orca', )
+
+repo_key_sha256 = "f49af3f11efddf20dffd70f5e3117b9976674167adca280e6b1932a0601b26f6"
